html
    height: 100%

body
    background: #173529
    padding: 0
    margin: 0
    font-family: Arial, Helvetica, sans-serif
    font-size: 14px
    color: #fff
    padding-bottom: 40px
    padding-top: 93px
    position: relative
    box-sizing: border-box
    min-height: 100%
    -webkit-font-smoothing: antialiased

ul
    margin: 0
    padding: 0
    list-style: none

a
    color: #2be6a4
    text-decoration: none
    &:hover
        color: #4ff9bd
